@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -22,7 +22,10 @@
 #include <stdio.h>
 #include <signal.h>
 #include <sys/ioctl.h>
+#include <sys/stat.h>
+#include <sys/wait.h>
 #include <linux/vt.h>
+#include <linux/kdev_t.h>
 #include "config.h"
 #include "fbterm.h"
 #include "fbshell.h"
@@ -33,13 +36,12 @@
 #include "input.h"
 #include "input_key.h"
 #include "mouse.h"
-#include "improxy.h"
 
 #ifdef HAVE_SIGNALFD
 // <sys/signalfd.h> offered by some systems has bug with g++
 #include "signalfd.h"
 
-sigset_t oldSigmask;
+static sigset_t oldSigmask;
 
 class SignalIo : public IoPipe {
 public:
@@ -74,7 +76,7 @@ static void signalHandler(s32 signo)
 static void pollSignal()
 {
 	if (!pendsigs) return;
-	
+
 	sig_atomic_t sigs = pendsigs;
 	pendsigs = 0;
 
@@ -103,7 +105,6 @@ FbTerm::~FbTerm()
 {
 	IoDispatcher::uninstance();
 	FbShellManager::uninstance();
-	ImProxy::uninstance();
 	Screen::uninstance();
 }
 
@@ -122,6 +123,10 @@ void FbTerm::init()
 #ifdef HAVE_SIGNALFD
 	sigset_t sigmask;
 	sigemptyset(&sigmask);
+
+#ifndef HAVE_EPOLL
+	sigaddset(&sigmask, SIGCHLD);
+#endif
 	sigaddset(&sigmask, SIGUSR1);
 	sigaddset(&sigmask, SIGUSR2);
 	sigaddset(&sigmask, SIGALRM);
@@ -133,6 +138,9 @@ void FbTerm::init()
 #else
 	sighandler_t sh = signalHandler;
 
+#ifndef HAVE_EPOLL
+	signal(SIGCHLD, sh);
+#endif
 	signal(SIGUSR1, sh);
 	signal(SIGUSR2, sh);
 	signal(SIGALRM, sh);
@@ -148,6 +156,17 @@ void FbTerm::init()
 void FbTerm::run()
 {
 	if (!mInit) return;
+
+	struct vt_stat vtstat;
+	ioctl(STDIN_FILENO, VT_GETSTATE, &vtstat);
+
+	struct stat ttystat;
+	fstat(STDIN_FILENO, &ttystat);
+
+	if (vtstat.v_active == MINOR(ttystat.st_rdev)) {
+		processSignal(SIGUSR2);
+	}
+
 	FbShellManager::instance()->createShell();
 
 	mRun = true;
@@ -185,6 +204,17 @@ void FbTerm::processSignal(u32 signo)
 		FbShellManager::instance()->switchVc(true);
 		break;
 
+#ifndef HAVE_EPOLL
+	case SIGCHLD:
+		if (mRun) {
+			s32 pid = waitpid(WAIT_ANY, 0, WNOHANG);
+			if (pid > 0) {
+				FbShellManager::instance()->childProcessExited(pid);
+			}
+		}
+		break;
+#endif
+
 	default:
 		break;
 	}
@@ -231,7 +261,9 @@ void FbTerm::processSysKey(u32 key)
 		break;
 
 	case CTRL_SPACE:
-		manager->toggleIm();
+		if (manager->activeShell()) {
+			manager->activeShell()->toggleIm();
+		}
 		break;
 
 	default:
@@ -241,9 +273,7 @@ void FbTerm::processSysKey(u32 key)
 
 void FbTerm::initChildProcess()
 {
-#ifndef HAVE_FS_CAPABILITY
-    setuid(getuid());
-#endif
+	setuid(getuid());
 
 #ifdef HAVE_SIGNALFD
 	sigprocmask(SIG_SETMASK, &oldSigmask, 0);
@@ -252,22 +282,18 @@ void FbTerm::initChildProcess()
 	signal(SIGPIPE, SIG_DFL);
 }
 
-#ifndef HAVE_FS_CAPABILITY
 u32 effective_uid;
-#endif
 
 int main(int argc, char **argv)
 {
-#ifndef HAVE_FS_CAPABILITY
 	effective_uid = geteuid();
 	seteuid(getuid());
-#endif
 
 	if (Config::instance()->parseArgs(argc, argv)) {
 		FbTerm::instance()->run();
 		FbTerm::uninstance();
 	}
-	
+
 	Config::uninstance();
 	return 0;
 }
