@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -40,7 +40,7 @@ protected:
 	void write(s8 *buf, u32 len);
 
 	virtual void readyRead(s8 *buf, u32 len) = 0;
-	virtual void ioEnd() {}
+	virtual void ioError(bool read, s32 err) {}
 
 private:
 	void translate(bool isread, s8 *buf, u32 len);
