@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
@@ -37,11 +37,12 @@ void waitChildProcessExit(s32 pid)
 {
 	if (pid < 0) return;
 
+	kill(pid, SIGTERM);
 	sched_yield();
 
 	s32 ret = waitpid(pid, 0, WNOHANG);
 	if (ret > 0 || (ret == -1 && errno == ECHILD)) return;
-	
+
 	for (u32 i = 10; i--;) {
 		timespec ts = { 0, 100 * 1000000UL };
 		nanosleep(&ts, 0);
@@ -49,7 +50,7 @@ void waitChildProcessExit(s32 pid)
 		ret = waitpid(pid, 0, WNOHANG);
 		if (ret > 0) break;
 	}
-		
+
 	if (ret <= 0) {
 		kill(pid, SIGKILL);
 		waitpid(pid, 0, 0);
@@ -59,7 +60,7 @@ void waitChildProcessExit(s32 pid)
 Shell::SelectedText Shell::mSelText;
 
 Shell::Shell()
-{	
+{
 	mPid = -1;
 }
 
@@ -69,7 +70,7 @@ Shell::~Shell()
 	waitChildProcessExit(mPid);
 }
 
-void Shell::createChildProcess()
+void Shell::createShellProcess(s8 **command)
 {
 	s32 fd;
 	mPid = forkpty(&fd, NULL, NULL, NULL);
@@ -77,16 +78,25 @@ void Shell::createChildProcess()
 	switch (mPid) {
 	case -1:
 		break;
-	
-	case 0: {  // child process
-		initChildProcess();
+
+	case 0:  // child process
+		initShellProcess();
 		setenv("TERM", "linux", 1);
 
-		struct passwd *userpd = getpwuid(getuid());
-		execlp(userpd->pw_shell, userpd->pw_shell, NULL);
+		if (command) {
+			execvp(command[0], command);
+		} else {
+			s8 *shell = getenv("SHELL");
+			if (shell) execlp(shell, shell, NULL);
+
+			struct passwd *userpd = getpwuid(getuid());
+			execlp(userpd->pw_shell, userpd->pw_shell, NULL);
+
+			execlp("/bin/sh", "/bin/sh", NULL);
+		}
+
 		exit(1);
 		break;
-	}
 
 	default:
 		setFd(fd);
@@ -136,12 +146,12 @@ void Shell::mouseInput(u16 x, u16 y, s32 type, s32 buttons)
 	s32 btn = buttons & MouseButtonMask;
 	s32 modifies = buttons & ModifyButtonMask;
 	u16 rtype = mode(MouseReport);
-	
+
 	if (btn && rtype != Wheel && (rtype == MouseNone || (modifies & ShiftButton))) {
 		textSelect(x, y, type, btn);
 		return;
 	}
-	
+
 	if (rtype == MouseNone) return;
 
 	s32 val = -1;
@@ -228,18 +238,18 @@ void Shell::middleTextSelect(u16 x, u16 y)
 {
 	u32 start = mSelState.start, end = mSelState.end;
 	u32 new_end = y * w() + x;
-	
+
 	bool dir_sel = (end >= start);
 	bool dir_new_sel = (new_end >= start);
-	
+
 	mSelState.end = new_end;
-	
+
 	if (dir_sel == dir_new_sel) {
 		bool dir_change = (new_end > end);
-		
+
 		u32 &pos = (dir_sel == dir_change) ? end : new_end;
 		CharAttr attr = charAttr(pos % w(), pos / w());
-		
+
 		if (dir_sel) {
 			if (attr.type == CharAttr::DoubleLeft) pos++;
 			pos++;
@@ -247,9 +257,9 @@ void Shell::middleTextSelect(u16 x, u16 y)
 			if (attr.type == CharAttr::DoubleRight) pos--;
 			pos--;
 		}
-		
+
 		bool dir_new_change = (new_end == end) ? dir_change : (new_end > end);
-		
+
 		if (dir_change == dir_new_change) {
 			inverseTextColor(end, new_end);
 		}
@@ -292,7 +302,7 @@ void Shell::endTextSelect()
 {
 	u32 start = mSelState.start, end = mSelState.end;
 	SWAP(start, end);
-	
+
 	u32 len = end - start + 1;
 	u16 buf[len];
 	s8 *text = new s8[len * 3];
@@ -319,7 +329,7 @@ void Shell::resetTextSelect()
 {
 	mSelState.selecting = false;
 	if (mSelState.color_inversed) {
-		mSelState.color_inversed = false;		
+		mSelState.color_inversed = false;
 		inverseTextColor(mSelState.start, mSelState.end);
 	}
 }
@@ -336,14 +346,14 @@ void Shell::autoTextSelect(u16 x, u16 y)
 		0xFF7FFFFF, /* latin-1 accented letters, not multiplication sign */
 		0xFF7FFFFF  /* latin-1 accented letters, not division sign */
 	};
-	
+
 	static bool inited = false;
 	if (!inited) {
 		inited = true;
-		
+
 		u8 chrs[32];
 		initWordChars((s8*)chrs, sizeof(chrs));
-		
+
 		for (u32 i = 0; chrs[i]; i++) {
 			if (chrs[i] > 0x7f || chrs[i] <= ' ') continue;
 			inwordLut[chrs[i] >> 5] |= 1 << (chrs[i] & 0x1f);
@@ -397,18 +407,18 @@ void Shell::inverseTextColor(u32 start, u32 end)
 	u16 sx, sy, ex, ey;
 	sx = start % w(), sy = start / w();
 	ex = end % w(), ey = end / w();
-	
+
 	inverse(sx, sy, ex, ey);
 
 	if (charAttr(sx, sy).type == CharAttr::DoubleRight) sx--;
 	if (charAttr(ex, ey).type == CharAttr::DoubleLeft) ex++;
 
 	requestUpdate(sx, sy, (sy == ey ? (ex + 1) : w()) - sx, 1);
-	
+
 	if (ey > sy + 1) {
 		requestUpdate(0, sy + 1, w(), ey - sy - 1);
 	}
-	
+
 	if (ey > sy) {
 		requestUpdate(0, ey, ex + 1, 1);
 	}
