@@ -1,5 +1,5 @@
 /*
- *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   Copyright © 2008-2009 dragchan <zgchan317@gmail.com>
  *   This file is part of FbTerm.
  *   based on GTerm by Timothy Miller <tim@techsource.com>
  *
@@ -57,32 +57,33 @@ VTerm::CharAttr VTerm::erase_char_attr()
 
 VTerm::ModeFlag::ModeFlag()
 {
-	bzero(this, sizeof(ModeFlag));
+	memset(this, 0, sizeof(ModeFlag));
 	auto_wrap = true;
 	cursor_visible = true;
 	autorepeat_key = true;
 }
 
 u16 VTerm::history_lines;
-const VTerm::StateOption *VTerm::hash_control_state[256];
-const VTerm::StateOption *VTerm::hash_esc_state[128];
-const VTerm::StateOption *VTerm::hash_square_state[128];
+u8 VTerm::control_map[MAX_CONTROL_CODE], VTerm::escape_map[NR_STATES][MAX_ESCAPE_CODE];
 
 void VTerm::init_state()
 {
-	#define INIT_STATE_TABLE(a) \
-	do { \
-		bzero(hash_##a, sizeof(hash_##a)); \
-		for (u32 i = 0; a[i].key != (u16)-1; i++) { \
-			for (u32 j = 0; j <= (u32)(a[i].key >> 8); j++) { \
-				hash_##a[(u32)(a[i].key & 0xff) + j] = a + i; \
-			} \
-		} \
-	} while (0)
+	for (u8 i = 1; control_sequences[i].code != (u16)-1; i++) {
+		control_map[control_sequences[i].code] = i;
+	}
 
-	INIT_STATE_TABLE(control_state);
-	INIT_STATE_TABLE(esc_state);
-	INIT_STATE_TABLE(square_state);
+	u8 state = ESnormal;
+	for (u8 i = 1; ; i++) {
+		if (escape_sequences[i].code == (u16)-1) {
+			state++;
+			if (state == NR_STATES) break;
+		} else {
+			u8 start = escape_sequences[i].code & 0xff;
+			for (u8 j = 0; j <= escape_sequences[i].code >> 8; j++) {
+				escape_map[state][start + j] = i;
+			}
+		}
+	}
 }
 
 VTerm::VTerm(u16 w, u16 h)
@@ -131,7 +132,7 @@ void VTerm::reset()
 	utf8 = true;
 	utf8_count = 0;
 //	g0_is_active = true;
-	normal_state = true;
+	esc_state = ESnormal;
 
 	pending_scroll = 0;
 	scroll_top = 0;
@@ -147,7 +148,7 @@ void VTerm::reset()
 	cur_halfbright_color = -1;
 
 	if (text) {
-		bzero(tab_stops, max_width / 8 + 1);
+		memset(tab_stops, 0, max_width / 8 + 1);
 		clear_area(0, 0, width - 1, height - 1);
 	}
 
@@ -164,7 +165,7 @@ void VTerm::resize(u16 w, u16 h)
 
 	if (new_max_width > max_width) {
 		s8 *new_tab_stops = new s8[new_max_width / 8 + 1];
-		bzero(new_tab_stops, new_max_width / 8 + 1);
+		memset(new_tab_stops, 0, new_max_width / 8 + 1);
 
 		if (tab_stops) {
 			memcpy(new_tab_stops, tab_stops, max_width / 8 + 1);
@@ -172,7 +173,7 @@ void VTerm::resize(u16 w, u16 h)
 		}
 		tab_stops = new_tab_stops;
 	} else if (w > width) {
-		
+
 	}
 
 	if (new_max_height > max_height) {
@@ -235,7 +236,7 @@ void VTerm::resize(u16 w, u16 h)
 	}
 
 	bool h_changed = false;
-	
+
 	if (h != height) {
 		h_changed = true;
 
@@ -253,11 +254,11 @@ void VTerm::resize(u16 w, u16 h)
 
 	width = w;
 	height = h;
-	
+
 	if (h_changed) {
 		historyChanged(visual_start_line, total_history_lines());
 	}
-	
+
 	scroll_bot = height - 1;
 	if (scroll_top >= height) scroll_top = 0;
 
@@ -274,7 +275,7 @@ void VTerm::resize(u16 w, u16 h)
 void VTerm::input(const u8 *buf, u32 count)
 {
 	if (!width) return;
-	
+
 	if (visual_start_line != total_history_lines()) {
 		historyDisplay(true, total_history_lines());
 		historyChanged(visual_start_line, total_history_lines());
@@ -295,7 +296,7 @@ void VTerm::input(const u8 *buf, u32 count)
 		rescan = 0;
 
 		/* Do no translation at all in control states */
-		if (!normal_state) {
+		if (esc_state != ESnormal) {
 			tc = c;
 		} else { // if (utf8 && !mode_flags.display_ctrl) {
 			rescan_last_byte:
@@ -387,11 +388,11 @@ void VTerm::input(const u8 *buf, u32 count)
 		 * direct-to-font zone in UTF-8 mode.
 		 */
 		bool ok = tc && (c >= 32 || !(mode_flags.display_ctrl ? ((CTRL_ALWAYS >> c) & 1) : (utf8 || ((CTRL_ACTION >> c) & 1))))
-				  && (c != 127 || mode_flags.display_ctrl)
-				  && (c != 128+27);
+					&& (c != 127 || mode_flags.display_ctrl)
+					&& (c != 128+27);
 
-		if (normal_state && ok) {
-			if (c == 0xfeff || (c >= 0x200b && c <= 0x200f)) { // zero width 
+		if ((esc_state == ESnormal) && ok) {
+			if (c == 0xfeff || (c >= 0x200b && c <= 0x200f)) { // zero width
 				continue;
 			}
 
@@ -502,44 +503,15 @@ void VTerm::do_normal_char()
 
 void VTerm::do_control_char()
 {
-	const StateOption *so = 0;
+	u8 index = (cur_char < MAX_CONTROL_CODE ? control_map[cur_char] : 0);
+	const Sequence *seq = control_sequences + index;
 
-	#define DO_ACTION() \
-	do { \
-		if (so->action) { \
-			(this->*(so->action))(); \
-		} \
-		 \
-		if (so->next_state == (StateOption*)-1) { \
-			normal_state = true; \
-		} else if (so->next_state) { \
-			normal_state = false; \
-			current_state = so->next_state; \
-		} \
-	} while (0)
-
-	if (cur_char < 256) so = hash_control_state[cur_char];
-	if (so) DO_ACTION();
-
-	if (so || normal_state) return;
-
-	if (current_state == esc_state || current_state == square_state) {
-		if (cur_char < 128) {
-			so = ((current_state == esc_state) ? hash_esc_state : hash_square_state)[cur_char];
-		}
-	} else {
-		for (const StateOption *cur = current_state; cur->key != (u16)-1; cur++) {
-			u32 key = cur->key & 0xff;
-			u32 same = cur->key >> 8;
-			if (cur_char >= key && cur_char <= (key + same)) {
-				so = cur;
-				break;
-			}
-		}
+	if (!index && esc_state != ESnormal && cur_char) {
+		seq = escape_sequences + (cur_char < MAX_ESCAPE_CODE ? escape_map[esc_state][cur_char] : 0);
 	}
 
-	if (so) DO_ACTION();
-	else normal_state = true;
+	if (seq->action) (this->*(seq->action))();
+	if (seq->next != ESkeep) esc_state = seq->next;
 }
 
 void VTerm::update()
@@ -616,10 +588,10 @@ void VTerm::expose(u16 x, u16 y, u16 w, u16 h)
 		bool dws[width];
 		u16 codes[width], num = 0;
 		u16 cur, start = startx;
-		
+
 		for (cur = startx; cur <= endx; cur++) {
 			if (attrs[yp + cur].type == CharAttr::DoubleRight) continue;
-		
+
 			if (attrs[yp + cur] != attr) {
 				attr.reverse ^= mode_flags.inverse_screen;
 				drawChars(attr, start, y, cur - start, num, codes, dws);
@@ -644,7 +616,7 @@ void VTerm::inverse(u16 sx, u16 sy, u16 ex, u16 ey)
 	if (charAttr(sx, sy).type == CharAttr::DoubleRight) sx--;
 	if (charAttr(ex, ey).type == CharAttr::DoubleLeft) ex++;
 	if (sy == ey && sx > ex) return;
-	
+
 	for (u16 y = sy; y <= ey; y++) {
 		u32 yp = get_line(y) * max_width;
 		u16 start = (y == sy) ? sx : 0;
@@ -694,18 +666,18 @@ void VTerm::scroll_region(u16 start_y, u16 end_y, s16 num)
 
 			linenumbers[y] = temp[takey];
 
-            if (!fast_scroll || clr) {
-                dirty_startx[y] = 0;
-                dirty_endx[y] = width-1;
-            } else {
-                dirty_startx[y] = temp_sx[takey];
-                dirty_endx[y] = temp_ex[takey];
-            }
- 
-            if (clr) {
-                clear_area(0, y, width - 1, y);
-            }
- 		}
+			if (!fast_scroll || clr) {
+				dirty_startx[y] = 0;
+				dirty_endx[y] = width-1;
+			} else {
+				dirty_startx[y] = temp_sx[takey];
+				dirty_endx[y] = temp_ex[takey];
+			}
+
+			if (clr) {
+				clear_area(0, y, width - 1, y);
+			}
+		}
 	}
 }
 
@@ -853,7 +825,7 @@ void VTerm::historyDisplay(bool absolute, s32 num)
 
 	if (visual_start_line > total_history_lines()) visual_start_line = total_history_lines();
 	if (visual_start_line == bak_line) return;
-	
+
 	modeChanged(CursorVisible);
 
 	bool accel_scroll = false;
@@ -881,7 +853,7 @@ void VTerm::historyDisplay(bool absolute, s32 num)
 	if (!accel_scroll) {
 		requestUpdate(0, 0, width, height);
 	}
-	
+
 	draw_cursor();
 }
 
